@@ -4,8 +4,10 @@ use std::io;
 use std::os::raw::c_char;
 use std::slice;
 
+use crate::errors::SourmashError;
+
+use crate::encodings::HashFunctions;
 use crate::signature::Signature;
-use crate::sketch::minhash::HashFunctions;
 use crate::sketch::Sketch;
 
 use crate::ffi::cmd::compute::SourmashComputeParameters;
@@ -165,15 +167,16 @@ ffi_fn! {
 unsafe fn signature_first_mh(ptr: *const SourmashSignature) -> Result<*mut SourmashKmerMinHash> {
     let sig = SourmashSignature::as_rust(ptr);
 
-    if let Some(item) = sig.signatures.get(0) {
-        if let Sketch::MinHash(mh) = item {
-          Ok(SourmashKmerMinHash::from_rust(mh.clone()))
-        } else {
-          unimplemented!()
-        }
-    } else {
-        // TODO: need to select the correct one
-        unimplemented!()
+    match sig.signatures.get(0) {
+        Some(Sketch::MinHash(mh)) => {
+            Ok(SourmashKmerMinHash::from_rust(mh.clone()))
+        },
+        Some(Sketch::LargeMinHash(mh_btree)) => {
+            Ok(SourmashKmerMinHash::from_rust(mh_btree.into()))
+        },
+        _ => Err(SourmashError::Internal {
+            message: "found unsupported sketch type".to_string()
+        }),
     }
 }
 }
