@@ -1,10 +1,25 @@
+// When the `wee_alloc` feature is enabled, use `wee_alloc` as the global
+// allocator.
+#[cfg(feature = "wee_alloc")]
+#[global_allocator]
+static ALLOC: wee_alloc::WeeAlloc = wee_alloc::WeeAlloc::INIT;
+
 use wasm_bindgen::prelude::*;
 
-use serde_json;
+use crate::cmd::ComputeParameters as _ComputeParameters;
+use crate::encodings::HashFunctions;
+use crate::signature::Signature as _Signature;
+use crate::signature::SigsTrait;
+use crate::sketch::minhash::KmerMinHash as _KmerMinHash;
 
-use crate::cmd::ComputeParameters;
-use crate::signature::{Signature, SigsTrait};
-use crate::sketch::minhash::{max_hash_for_scaled, HashFunctions, KmerMinHash};
+#[wasm_bindgen]
+pub struct KmerMinHash(_KmerMinHash);
+
+#[wasm_bindgen]
+pub struct Signature(_Signature);
+
+#[wasm_bindgen]
+pub struct ComputeParameters(_ComputeParameters);
 
 #[wasm_bindgen]
 impl KmerMinHash {
@@ -19,12 +34,6 @@ impl KmerMinHash {
         scaled: u32,
         track_abundance: bool,
     ) -> KmerMinHash {
-        let max_hash = if num != 0 {
-            0
-        } else {
-            max_hash_for_scaled(scaled as u64).unwrap()
-        };
-
         // TODO: at most one of (prot, dayhoff, hp) should be true
 
         let hash_function = if dayhoff {
@@ -37,25 +46,26 @@ impl KmerMinHash {
             HashFunctions::murmur64_DNA
         };
 
-        KmerMinHash::new(
-            num,
+        KmerMinHash(_KmerMinHash::new(
+            scaled as u64,
             ksize,
             hash_function,
             seed as u64,
-            max_hash,
             track_abundance,
-        )
+            num,
+        ))
     }
 
     #[wasm_bindgen]
     pub fn add_sequence_js(&mut self, buf: &str) {
-        self.add_sequence(buf.as_bytes(), true)
+        self.0
+            .add_sequence(buf.as_bytes(), true)
             .expect("Error adding sequence");
     }
 
     #[wasm_bindgen]
     pub fn to_json(&mut self) -> String {
-        serde_json::to_string(self).unwrap()
+        serde_json::to_string(&self.0).unwrap()
     }
 }
 
@@ -63,8 +73,13 @@ impl KmerMinHash {
 impl ComputeParameters {
     #[wasm_bindgen(constructor)]
     pub fn new_with_params() -> ComputeParameters {
-        let params = ComputeParameters::default();
-        params
+        let params = _ComputeParameters::default();
+        ComputeParameters(params)
+    }
+
+    #[wasm_bindgen]
+    pub fn set_ksizes(&mut self, ksizes: Vec<u32>) {
+        self.0.set_ksizes(ksizes);
     }
 }
 
@@ -74,18 +89,28 @@ impl Signature {
     pub fn new_from_params(params: &ComputeParameters) -> Signature {
         //let params = ComputeParameters::default();
 
-        Signature::from_params(&params)
+        Signature(_Signature::from_params(&params.0))
     }
 
     #[wasm_bindgen]
     pub fn add_sequence_js(&mut self, buf: &str) {
-        self.add_sequence(buf.as_bytes(), true)
+        self.0
+            .add_sequence(buf.as_bytes(), true)
             .expect("Error adding sequence");
     }
 
+    #[wasm_bindgen]
+    pub fn add_from_file(&mut self, fp: web_sys::File) {
+        unimplemented!()
+    }
+
     #[wasm_bindgen]
     pub fn to_json(&mut self) -> String {
-        serde_json::to_string(self).unwrap()
+        serde_json::to_string(&self.0).unwrap()
+    }
+
+    pub fn size(&self) -> usize {
+        self.0.size()
     }
 }
 
