@@ -66,7 +66,7 @@ def subparser(subparsers):
         help='What percentage of internal nodes will not be saved; ranges '
         'from 0.0 (save all nodes) to 1.0 (no nodes saved)'
     )
-    add_ksize_arg(subparser, 31)
+    add_ksize_arg(subparser)
     add_moltype_args(subparser)
     add_picklist_args(subparser)
     add_scaled_arg(subparser, 0)
