@@ -23,11 +23,12 @@ Please see the 'tax metagenome' documentation for more details:
 
 import sourmash
 from sourmash.logging import notify, print_results, error
+from sourmash.cli.utils import add_rank_arg, check_rank, check_tax_outputs
+
 
 
 def subparser(subparsers):
     subparser = subparsers.add_parser('metagenome',
-                                      aliases=['summarize'],
                                       usage=usage)
     subparser.add_argument(
         '-g', '--gather-csv', action="extend", nargs='*', default = [],
@@ -68,29 +69,34 @@ def subparser(subparsers):
     )
     subparser.add_argument(
         '-F', '--output-format', default=[], nargs='*', action="extend",
-        choices=["human", "csv_summary", "krona", "lineage_summary",  "kreport"],
+        choices=["human", "csv_summary", "krona", "lineage_summary", "kreport", "lingroup", "bioboxes"],
         help='choose output format(s)',
     )
-    subparser.add_argument(
-        '-r', '--rank', choices=['strain','species', 'genus', 'family', 'order', 'class', 'phylum', 'superkingdom'],
-        help='For non-default output formats: Summarize genome taxonomy at this rank and above. Note that the taxonomy CSV must contain lineage information at this rank.'
-    )
     subparser.add_argument(
         '-f', '--force', action = 'store_true',
         help='continue past errors in taxonomy database loading',
     )
+    subparser.add_argument(
+        '--lins', '--lin-taxonomy', action='store_true', default=False,
+        help="use LIN taxonomy in place of standard taxonomic ranks.  Note that the taxonomy CSV must contain 'lin' lineage information."
+    )
+    subparser.add_argument(
+        '--lingroup', '--lingroups', metavar='FILE', default=None,
+        help="CSV containing 'name', 'lin' columns, where 'lin' is the lingroup prefix. Will produce a 'lingroup' report containing taxonomic summarization for each group."
+    )
+    add_rank_arg(subparser)
 
 def main(args):
     import sourmash
-    if not args.gather_csv and not args.from_file:
-        raise ValueError(f"No gather CSVs found! Please input via '-g' or '--from-file'.")
-    if len(args.output_format) > 1:
-        if args.output_base == "-":
-            raise TypeError(f"Writing to stdout is incompatible with multiple output formats {args.output_format}")
-    if not args.rank:
-        if any(x in ["krona", "lineage_summary"] for x in args.output_format):
-            raise ValueError(f"Rank (--rank) is required for krona and lineage_summary output formats.")
-    if not args.output_format:
-        # change to "human" for 5.0
-        args.output_format = ["csv_summary"]
+    try:
+        if not args.gather_csv and not args.from_file:
+            raise ValueError(f"No gather CSVs found! Please input via '-g' or '--from-file'.")
+        if args.rank:
+            args.rank = check_rank(args)
+        args.output_format = check_tax_outputs(args, rank_required = ['krona', 'lineage_summary'], incompatible_with_lins = ['bioboxes', 'kreport'], use_lingroup_format=True)
+
+    except ValueError as exc:
+        error(f"ERROR: {str(exc)}")
+        import sys; sys.exit(-1)
+
     return sourmash.tax.__main__.metagenome(args)
