@@ -2,7 +2,7 @@
 
 from .lca_db import LCA_Database
 from .lca_utils import (taxlist, zip_lineage, build_tree, find_lca,
-                        gather_assignments, LineagePair, display_lineage,
+                        gather_assignments, display_lineage,
                         count_lca_for_assignments)
 
 from .command_index import index
