@@ -103,17 +103,6 @@ filter_null = lambda x: 'unassigned' if x is None or x.strip() in \
 null_names = set(['[Blank]', 'na', 'null'])
 
 
-def make_lineage(lineage_str):
-    "Turn a ; or ,-separated set of lineages into a tuple of LineagePair objs."
-    lin = lineage_str.split(';')
-    if len(lin) == 1:
-        lin = lineage.split(',')
-    lin = [ LineagePair(rank, n) for (rank, n) in zip(taxlist(), lin) ]
-    lin = tuple(lin)
-
-    return lin
-
-
 def build_tree(assignments, initial=None):
     """
     Builds a tree of dictionaries from lists of LineagePair objects
@@ -245,6 +234,7 @@ def pop_to_rank(lin, rank):
 
 def make_lineage(lineage):
     "Turn a ; or ,-separated set of lineages into a tuple of LineagePair objs."
+    from sourmash.tax.tax_utils import LineagePair
     lin = lineage.split(';')
     if len(lin) == 1:
         lin = lineage.split(',')
