@@ -6,7 +6,7 @@ class MinHash - core MinHash class.
 class FrozenMinHash - read-only MinHash class.
 """
 from __future__ import unicode_literals, division
-from .distance_utils import jaccard_to_distance, containment_to_distance, set_size_chernoff
+from .distance_utils import jaccard_to_distance, containment_to_distance, set_size_exact_prob
 from .logging import notify
 
 import numpy as np
@@ -745,19 +745,6 @@ class MinHash(RustObject):
         return self._methodcall(lib.kmerminhash_is_compatible, other._get_objptr())
 
     def contained_by(self, other, downsample=False):
-        """
-        Calculate how much of self is contained by other.
-        """
-        if not (self.scaled and other.scaled):
-            raise TypeError("Error: can only calculate containment for scaled MinHashes")
-        if not len(self):
-            return 0.0
-        return self.count_common(other, downsample) / len(self)
-        # with bias factor
-        #return self.count_common(other, downsample) / (len(self) * (1- (1-1/self.scaled)^(len(self)*self.scaled)))
-
-
-    def contained_by_debiased(self, other, downsample=False):
         """
         Calculate how much of self is contained by other.
         """
@@ -769,11 +756,11 @@ class MinHash(RustObject):
         total_denom = float(denom * self.scaled) # would be better if hll estimate - see #1798
         bias_factor = 1.0 - (1.0 - 1.0/self.scaled) ** total_denom
         containment = self.count_common(other, downsample) / (denom * bias_factor)
-        # debiasing containment can lead to vals outside of 0-1 range!?
+        # debiasing containment can lead to vals outside of 0-1 range. constrain.
         if containment >= 1:
-            return 1
+            return 1.0
         elif containment <= 0:
-            return 0
+            return 0.0
         else:
             return containment
 
@@ -789,8 +776,8 @@ class MinHash(RustObject):
             scaled = max(self_mh.scaled, other_mh.scaled)
             self_mh = self.downsample(scaled=scaled)
             other_mh = other.downsample(scaled=scaled)
-        containment = self_mh.contained_by_debiased(other_mh) # recalc debiased containment
-        #containment = self_mh.contained_by(other_mh) # recalc debiased containment
+        if containment is None:
+            containment = self_mh.contained_by(other_mh)
         n_kmers = len(self_mh) * scaled # would be better if hll estimate - see #1798
 
         c_aniresult = containment_to_distance(containment, self_mh.ksize, self_mh.scaled,
@@ -801,21 +788,7 @@ class MinHash(RustObject):
             c_aniresult.size_is_inaccurate = True
         return c_aniresult
 
-
     def max_containment(self, other, downsample=False):
-        """
-        Calculate maximum containment.
-        """
-        if not (self.scaled and other.scaled):
-            raise TypeError("Error: can only calculate containment for scaled MinHashes")
-        min_denom = min((len(self), len(other)))
-        if not min_denom:
-            return 0.0
-
-        return self.count_common(other, downsample) / min_denom
-
-
-    def max_containment_debiased(self, other, downsample=False):
         """
         Calculate maximum containment.
         """
@@ -827,11 +800,11 @@ class MinHash(RustObject):
         total_denom =  float(min_denom * self.scaled) # would be better if hll estimate - see #1798
         bias_factor = 1.0 - (1.0 - 1.0/self.scaled) ** total_denom
         max_containment = self.count_common(other, downsample) / (min_denom * bias_factor)
-        # debiasing containment can lead to vals outside of 0-1 range!?
+        # debiasing containment can lead to vals outside of 0-1 range. constrain.
         if max_containment >= 1:
-            return 1
+            return 1.0
         elif max_containment <= 0:
-            return 0
+            return 0.0
         else:
             return max_containment
 
@@ -846,7 +819,8 @@ class MinHash(RustObject):
             scaled = max(self_mh.scaled, other_mh.scaled)
             self_mh = self.downsample(scaled=scaled)
             other_mh = other.downsample(scaled=scaled)
-        max_containment = self_mh.max_containment_debiased(other_mh) #recalc debiased max containment
+        if max_containment is None:
+            max_containment = self_mh.max_containment(other_mh)
         min_n_kmers = min(len(self_mh), len(other_mh))
         n_kmers = min_n_kmers * scaled  # would be better if hll estimate - see #1798
 
@@ -871,19 +845,6 @@ class MinHash(RustObject):
 
         return (c1 + c2)/2
 
-    def avg_containment_debiased(self, other, downsample=False):
-        """
-        Calculate average containment, debiased.
-        Note: this is average of the containments, *not* count_common/ avg_denom
-        """
-        if not (self.scaled and other.scaled):
-            raise TypeError("Error: can only calculate containment for scaled MinHashes")
-
-        c1 = self.contained_by_debiased(other, downsample)
-        c2 = other.contained_by_debiased(self, downsample)
-
-        return (c1 + c2)/2
-
     def avg_containment_ani(self, other, *, downsample=False, prob_threshold=1e-3):
         """
         Calculate average containment ANI.
@@ -1051,7 +1012,7 @@ class MinHash(RustObject):
         if any([not (0 <= relative_error <= 1), not (0 <= confidence <= 1)]):
             raise ValueError("Error: relative error and confidence values must be between 0 and 1.")
         # to do: replace unique_dataset_hashes with HLL estimation when it gets implemented 
-        probability = set_size_chernoff(self.unique_dataset_hashes, self.scaled, relative_error=relative_error)
+        probability = set_size_exact_prob(self.unique_dataset_hashes, self.scaled, relative_error=relative_error)
         return probability >= confidence
 
 
