@@ -321,7 +321,7 @@ class SearchResult(BaseResult):
         if self.cmp_scaled is None:
             raise TypeError("Error: ANI can only be estimated from scaled signatures.")
         if self.searchtype == SearchType.CONTAINMENT:
-            self.cmp.estimate_ani_from_mh1_containment_in_mh2()
+            self.cmp.estimate_ani_from_mh1_containment_in_mh2(containment = self.similarity)
             self.ani = self.cmp.ani_from_mh1_containment_in_mh2
             if self.estimate_ani_ci:
                 self.ani_low = self.cmp.ani_from_mh1_containment_in_mh2_low
@@ -551,7 +551,7 @@ def format_bp(bp):
     "Pretty-print bp information."
     bp = float(bp)
     if bp < 500:
-        return '{:.0f} bp '.format(bp)
+        return '{:.0f} bp'.format(bp)
     elif bp <= 500e3:
         return '{:.1f} kbp'.format(round(bp / 1e3, 1))
     elif bp < 500e6:
@@ -783,12 +783,11 @@ class GatherDatabases:
         new_query_mh.remove_many(found_mh)
         new_query = SourmashSignature(new_query_mh)
 
-        # compute weighted_missed for remaining query hashes
+        # compute weighted information for remaining query hashes
         query_hashes = set(query_mh.hashes) - set(found_mh.hashes)
         n_weighted_missed = sum((orig_query_abunds[k] for k in query_hashes))
         n_weighted_missed += self.noident_query_sum_abunds
         sum_weighted_found = sum_abunds - n_weighted_missed
-        weighted_missed = n_weighted_missed / sum_abunds
 
         # build a GatherResult
         result = GatherResult(self.orig_query, best_match,
@@ -809,7 +808,7 @@ class GatherDatabases:
         self.query = new_query
         self.orig_query_mh = orig_query_mh
 
-        return result, weighted_missed
+        return result
 
 
 ###
