@@ -75,6 +75,10 @@ def lca_db_format(request):
 def manifest_db_format(request):
     return request.param
 
+@pytest.fixture(params=['sig', 'sig.gz', 'zip', '.d/', '.sqldb'])
+def sig_save_extension(request):
+    return request.param
+
 
 # --- BEGIN - Only run tests using a particular fixture --- #
 # Cribbed from: http://pythontesting.net/framework/pytest/pytest-run-tests-using-particular-fixture/
